﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SharpGL;
using SharpGL.SceneGraph;

namespace LindenmayerTool
{
    // Abstract class for holding stored pieces of geometry to be rendered/exported
    public abstract class DrawObject
    {
        public enum DO_Type { Line, PointSet, Prefab };
        public Nullable<Vertex> Highlight = null;
        public abstract void Render(OpenGL Renderer);
        public abstract void Render(OpenGL Renderer, Vertex Highlight);
        public void Render(OpenGL Renderer, int curTime)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            // if the associated audio event is being played, highlight us.
            if (curTime >= DrawTime && curTime <= DrawTime + (ls.StepSize * ls.StepTimeScale))
                Render(Renderer, Highlight ?? new Vertex(1.0f, 0.0f, 0.0f));
            else
                Render(Renderer);
        }

        // Time (in millis from start) that an audio event is associated with this DrawObject
        public int DrawTime;

        // Stored lines
        public class DO_Line : DrawObject
        {
            public Vertex P1     = new Vertex();
            public Vertex P2     = new Vertex();
            public Vertex Colour = new Vertex();
            public float Width   = 1.0f;

            public override void Render(OpenGL Renderer) 
            { 
                Renderer.LineWidth(Width);
                Renderer.Begin(OpenGL.GL_LINES);
                    Renderer.Color(Colour.X, Colour.Y, Colour.Z);
                    Renderer.Vertex(P1.X, P1.Y, P1.Z);
                    Renderer.Vertex(P2.X, P2.Y, P2.Z);
                Renderer.End();
            }
            public override void Render(OpenGL Renderer, Vertex Highlight) 
            {
                Renderer.LineWidth(Width);
                Renderer.Begin(OpenGL.GL_LINES);
                    Renderer.Color(Highlight.X, Highlight.Y, Highlight.Z);
                    Renderer.Vertex(P1.X, P1.Y, P1.Z);
                    Renderer.Vertex(P2.X, P2.Y, P2.Z);
                Renderer.End();
            }
        }


        // TODO : Extensions for in-system defined 3D objects
        public class DO_PointSet : DrawObject
        {
            public override void Render(OpenGL Renderer) { }
            public override void Render(OpenGL Renderer, Vertex Highlight) { }
        }
        public class DO_Prefab : DrawObject
        {
            public override void Render(OpenGL Renderer) { }
            public override void Render(OpenGL Renderer, Vertex Highlight) { }
        }

    }
}
