﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;
using System.ComponentModel;

namespace LindenmayerTool
{
    // Some stuff to allow for procedural listing of enumeration options,
    // instead of having me hardcode all the options, or listing them with ugly internal names.

    // Enum helper class
    // <http://stackoverflow.com/questions/6145888/how-to-bind-an-enum-to-a-combobox-control-in-wpf>

    // ValueDescription tuple
    //<http://code.google.com/p/messagingwpf/source/browse/trunk/src/StockTraderRI/StockTraderRI.Modules.Position/Orders/ValueDescription.cs?r=4>

    
    public class ValueDescription
    {
        public ValueDescription()
        {
        }

        public ValueDescription(Enum value, string description)
        {
            Value = value;
            Description = description;
        }
        public Enum Value { get; set; }
        public string Description { get; set; }
        public override string ToString()
        {
            return Description;
        }
    }


    public static class EnumHelper
    {
        /// <summary>
        /// Gets the description of a specific enum value.
        /// </summary>
        public static string Description(this Enum eValue)
        {
            var nAttributes = eValue.GetType().GetField(eValue.ToString()).GetCustomAttributes(typeof(DescriptionAttribute), false);

            if (!nAttributes.Any())
            {
                TextInfo oTI = CultureInfo.CurrentCulture.TextInfo;
                return oTI.ToTitleCase(oTI.ToLower(eValue.ToString().Replace("_", " ")));
            }

            return (nAttributes.First() as DescriptionAttribute).Description;
        }

        /// <summary>
        /// Returns an enumerable collection of all values and descriptions for an enum type.
        /// </summary>
        public static IEnumerable<ValueDescription> GetAllValuesAndDescriptions<TEnum>() where TEnum : struct, IConvertible, IComparable, IFormattable
        {
            if (!typeof(TEnum).IsEnum)
                throw new ArgumentException("TEnum must be an Enumeration type");

            return from e in Enum.GetValues(typeof(TEnum)).Cast<Enum>()
                   select new ValueDescription() { Value = e, Description = e.Description() };
        }
    }
}
