﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.IO;

namespace LindenmayerTool
{
    /// <summary>
    /// Interaction logic for MusicExport.xaml
    /// </summary>
    public partial class MusicMenu : Window
    {
        
        // CSound Paths
        public static String CS_Orc_File = "*.orc";
        public static String CS_Sco_Prefix_File = "*.sco";
        public static String CS_Bin_File = "csound.exe";


        private static MusicMenu _menu = null;
        public static MusicMenu I()
        {
            if (_menu == null) _menu = new MusicMenu();
            if (!_menu.IsLoaded) _menu = new MusicMenu();
            return _menu;
        }
        

        public MusicMenu()
        {
            InitializeComponent();
            ComboBoxInits();
            GetFileDefaults();
            GetLSysDefaults();
        }

        // Try to find the example files
        public void GetFileDefaults()
        {
            String cur = Environment.CurrentDirectory;
            String f_orc = "Example_Orchestra1.orc";
            String f_sco = "Example_ScorePrefix.sco";
            String back = ".." + Path.DirectorySeparatorChar;
            String egs = "Examples"+Path.DirectorySeparatorChar+"Sound";

            String[] OrcPaths = new String[] {
                Path.Combine(cur,egs,f_orc),
                Path.Combine(cur,back+egs,f_orc),
                Path.Combine(cur,back+back+egs,f_orc),
                Path.Combine(cur,back+back+back+egs,f_orc)
            };
            String[] ScoPaths = new String[] {
                Path.Combine(cur,egs,f_sco),
                Path.Combine(cur,back+egs,f_sco),
                Path.Combine(cur,back+back+egs,f_sco),
                Path.Combine(cur,back+back+back+egs,f_sco)
            };

            foreach(var p in OrcPaths)
                if (File.Exists(p))
                {
                    CS_Orc_File = Path.GetFullPath(p);
                    break;
                }
            foreach (var p in ScoPaths)
                if (File.Exists(p))
                {
                    CS_Sco_Prefix_File = Path.GetFullPath(p);
                    break;
                }

            this.CS_Path_Orc.Text = CS_Orc_File;
            this.CS_Path_Sco.Text = CS_Sco_Prefix_File;
            this.CS_Path_Bin.Text = CS_Bin_File;
        }

        // Update all the combo boxes with various Enum type enumerations
        // Select current values.
        private void ComboBoxInits()
        {
            var ls = (LSystem_Musical)(LSystem.I());

            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.TimeMetric>())
            {
                this.TimingOption.Items.Add(i);
                if (ls.TimingMetric == (LSystem_Musical.TimeMetric)i.Value)
                    TimingOption.SelectedItem = i;
            }

            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.NoteStepCreationMetric>())
            {
                this.NoteCreateTimeOption.Items.Add(i);
                if (ls.NoteStepCreateMetric == (LSystem_Musical.NoteStepCreationMetric)i.Value)
                    NoteCreateTimeOption.SelectedItem = i;
            }

            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.AdjustmentType>())
            {
                this.PitchAdjustType.Items.Add(i);
                if (ls.PitchAdjustmentType == (LSystem_Musical.AdjustmentType)i.Value)
                    PitchAdjustType.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.AdjustmentType>())
            {
                this.DurationAdjustType.Items.Add(i);
                if (ls.DurationAdjustmentType == (LSystem_Musical.AdjustmentType)i.Value)
                    DurationAdjustType.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.AdjustmentType>())
            {
                this.VolumeAdjustType.Items.Add(i);
                if (ls.VolumeAdjustmentType == (LSystem_Musical.AdjustmentType)i.Value)
                    VolumeAdjustType.SelectedItem = i;
            }

            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.AdjustmentType>())
            {
                this.Param1AdjustType.Items.Add(i);
                if (ls.ParamC1AdjustmentType == (LSystem_Musical.AdjustmentType)i.Value)
                    Param1AdjustType.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.AdjustmentType>())
            {
                this.Param2AdjustType.Items.Add(i);
                if (ls.ParamC2AdjustmentType == (LSystem_Musical.AdjustmentType)i.Value)
                    Param2AdjustType.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.AdjustmentType>())
            {
                this.Param3AdjustType.Items.Add(i);
                if (ls.ParamC3AdjustmentType == (LSystem_Musical.AdjustmentType)i.Value)
                    Param3AdjustType.SelectedItem = i;
            }

            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.IncrementOrAxis>())
            {
                this.PitchSource.Items.Add(i);
                if (ls.PitchDeterminationType == (LSystem_Musical.IncrementOrAxis)i.Value)
                    PitchSource.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.IncrementOrAxis>())
            {
                this.DurationSource.Items.Add(i);
                if (ls.DurationDeterminationType == (LSystem_Musical.IncrementOrAxis)i.Value)
                    DurationSource.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.IncrementOrAxis>())
            {
                this.VolumeSource.Items.Add(i);
                if (ls.VolumeDeterminationType == (LSystem_Musical.IncrementOrAxis)i.Value)
                    VolumeSource.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.IncrementOrAxis>())
            {
                this.Param1Source.Items.Add(i);
                if (ls.ParamC1DeterminationType == (LSystem_Musical.IncrementOrAxis)i.Value)
                    Param1Source.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.IncrementOrAxis>())
            {
                this.Param2Source.Items.Add(i);
                if (ls.ParamC2DeterminationType == (LSystem_Musical.IncrementOrAxis)i.Value)
                    Param2Source.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<LSystem_Musical.IncrementOrAxis>())
            {
                this.Param3Source.Items.Add(i);
                if (ls.ParamC3DeterminationType == (LSystem_Musical.IncrementOrAxis)i.Value)
                    Param3Source.SelectedItem = i;
            }
        }

        // Update form fields with current values.
        private void GetLSysDefaults()
        {
            var ls = (LSystem_Musical)(LSystem.I());

            this.NoteTimeScale.Text   = ls.StepTimeScale.ToString();
            this.PitchAdjust.Text     = ls.PitchAdjustment.ToString();
            this.PitchInitial.Text    = ls.PitchDefault.ToString();
            this.DurationAdjust.Text  = ls.DurationAdjustment.ToString();
            this.DurationInitial.Text = ls.DurationDefault.ToString();
            this.VolumeAdjust.Text    = ls.VolumeAdjustment.ToString();
            this.VolumeInitial.Text   = ls.VolumeDefault.ToString();

            this.Param1Adjust.Text  = ls.ParamC1Adjustment.ToString();
            this.Param1Initial.Text = ls.ParamC1Default.ToString();
            this.Param2Adjust.Text  = ls.ParamC2Adjustment.ToString();
            this.Param2Initial.Text = ls.ParamC2Default.ToString();
            this.Param3Adjust.Text  = ls.ParamC3Adjustment.ToString();
            this.Param3Initial.Text = ls.ParamC3Default.ToString();

            this.useAbsoluteAxisVals.IsChecked = ls.isAxisValuesAbsolute;
        }

        private void Button_Click_Apply(object sender, RoutedEventArgs e)
        {
            this.Hide();
        }

        private void Button_Click_Cancel(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void Button_Click_CS_Path_Orc(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog fileDialog = new Microsoft.Win32.OpenFileDialog()
            {
                Filter = "CSound Orchestra File|*.orc|All Files (*.*)|*.*",
                InitialDirectory = System.Environment.CurrentDirectory
            };
            bool? res = fileDialog.ShowDialog();
            if (res == null || res.Value == false) return;
            string filename = fileDialog.FileName;
            CS_Orc_File = filename;
            this.CS_Path_Orc.Text = filename;
        }

        private void Button_Click_CS_Path_Sco(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog fileDialog = new Microsoft.Win32.OpenFileDialog()
            {
                Filter = "CSound Score File|*.sco|All Files (*.*)|*.*",
                InitialDirectory = System.Environment.CurrentDirectory
            };
            bool? res = fileDialog.ShowDialog();
            if (res == null || res.Value == false) return;
            string filename = fileDialog.FileName;
            CS_Sco_Prefix_File = filename;
            this.CS_Path_Sco.Text = filename;
        }

        private void Button_Click_CS_Path_Bin(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog fileDialog = new Microsoft.Win32.OpenFileDialog()
            {
                Filter = "CSound Executable|csound.exe|All Files (*.*)|*.*",
                InitialDirectory = System.Environment.CurrentDirectory
            };
            bool? res = fileDialog.ShowDialog();
            if (res == null || res.Value == false) return;
            string filename = fileDialog.FileName;
            CS_Bin_File = filename;
            this.CS_Path_Bin.Text = filename;
        }

        private void Combo_Change_Timing(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.TimingMetric = (LSystem_Musical.TimeMetric)(((ValueDescription)(this.TimingOption.SelectedItem)).Value);
        }

        private void Combo_Change_NoteCreateTime(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.NoteStepCreateMetric = (LSystem_Musical.NoteStepCreationMetric)(((ValueDescription)(this.NoteCreateTimeOption.SelectedItem)).Value);
        }

        private void Textbox_TextChanged_NoteTimeScale(object sender, TextChangedEventArgs e)
        {
            String s = this.NoteTimeScale.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.StepTimeScale;
            if (float.TryParse(s, out v))
                ls.StepTimeScale = v;
            else
                this.NoteTimeScale.Text = ls.StepTimeScale.ToString();
        }

        private void Textbox_TextChanged_PitchInitial(object sender, TextChangedEventArgs e)
        {
            String s = this.PitchInitial.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.PitchDefault;
            if (float.TryParse(s, out v))
                ls.PitchDefault = v;
            else
                this.PitchInitial.Text = ls.PitchDefault.ToString();
        }

        private void Textbox_TextChanged_PitchAdjust(object sender, TextChangedEventArgs e)
        {
            String s = this.PitchAdjust.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.PitchAdjustment;
            if (float.TryParse(s, out v))
                ls.PitchAdjustment = v;
            else
                this.PitchAdjust.Text = ls.PitchAdjustment.ToString();
        }

        private void Combo_Change_PitchAdjustType(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.PitchAdjustmentType = (LSystem_Musical.AdjustmentType)(((ValueDescription)(this.PitchAdjustType.SelectedItem)).Value);
        }

        private void Textbox_TextChanged_DurationInitial(object sender, TextChangedEventArgs e)
        {
            String s = this.DurationInitial.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.DurationDefault;
            if (float.TryParse(s, out v))
                ls.DurationDefault = v;
            else
                this.DurationInitial.Text = ls.DurationDefault.ToString();
        }

        private void Textbox_TextChanged_DurationAdjust(object sender, TextChangedEventArgs e)
        {
            String s = this.DurationAdjust.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.DurationAdjustment;
            if (float.TryParse(s, out v))
                ls.DurationAdjustment = v;
            else
                this.DurationAdjust.Text = ls.DurationAdjustment.ToString();
        }

        private void Combo_Change_DurationAdjustType(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.DurationAdjustmentType = (LSystem_Musical.AdjustmentType)(((ValueDescription)(this.DurationAdjustType.SelectedItem)).Value);
        }

        private void Textbox_TextChanged_VolumeInitial(object sender, TextChangedEventArgs e)
        {
            String s = this.VolumeInitial.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.VolumeDefault;
            if (float.TryParse(s, out v))
                ls.VolumeDefault = v;
            else
                this.VolumeInitial.Text = ls.VolumeDefault.ToString();
        }

        private void Textbox_TextChanged_VolumeAdjust(object sender, TextChangedEventArgs e)
        {
            String s = this.VolumeAdjust.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.VolumeAdjustment;
            if (float.TryParse(s, out v))
                ls.VolumeAdjustment = v;
            else
                this.VolumeAdjust.Text = ls.VolumeAdjustment.ToString();
        }

        private void Combo_Change_VolumeAdjustType(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.VolumeAdjustmentType = (LSystem_Musical.AdjustmentType)(((ValueDescription)(this.VolumeAdjustType.SelectedItem)).Value);
        }

        private void Textbox_TextChanged_Param1Initial(object sender, TextChangedEventArgs e)
        {
            String s = this.Param1Initial.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.ParamC1Default;
            if (float.TryParse(s, out v))
                ls.ParamC1Default = v;
            else
                this.Param1Initial.Text = ls.ParamC1Default.ToString();
        }

        private void Textbox_TextChanged_Param1Adjust(object sender, TextChangedEventArgs e)
        {
            String s = this.Param1Adjust.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.ParamC1Adjustment;
            if (float.TryParse(s, out v))
                ls.ParamC1Adjustment = v;
            else
                this.Param1Adjust.Text = ls.ParamC1Adjustment.ToString();
        }

        private void Combo_Change_Param1AdjustType(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.ParamC1AdjustmentType = (LSystem_Musical.AdjustmentType)(((ValueDescription)(this.Param1AdjustType.SelectedItem)).Value);
        }

        private void Textbox_TextChanged_Param2Initial(object sender, TextChangedEventArgs e)
        {
            String s = this.Param2Initial.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.ParamC2Default;
            if (float.TryParse(s, out v))
                ls.ParamC2Default = v;
            else
                this.Param2Initial.Text = ls.ParamC2Default.ToString();
        }

        private void Textbox_TextChanged_Param2Adjust(object sender, TextChangedEventArgs e)
        {
            String s = this.Param2Adjust.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.ParamC2Adjustment;
            if (float.TryParse(s, out v))
                ls.ParamC2Adjustment = v;
            else
                this.Param2Adjust.Text = ls.ParamC2Adjustment.ToString();
        }

        private void Combo_Change_Param2AdjustType(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.ParamC2AdjustmentType = (LSystem_Musical.AdjustmentType)(((ValueDescription)(this.Param2AdjustType.SelectedItem)).Value);
        }

        private void Textbox_TextChanged_Param3Initial(object sender, TextChangedEventArgs e)
        {
            String s = this.Param3Initial.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.ParamC3Default;
            if (float.TryParse(s, out v))
                ls.ParamC3Default = v;
            else
                this.Param3Initial.Text = ls.ParamC3Default.ToString();
        }

        private void Textbox_TextChanged_Param3Adjust(object sender, TextChangedEventArgs e)
        {
            String s = this.Param3Adjust.Text;
            var ls = (LSystem_Musical)(LSystem.I());
            float v = ls.ParamC3Adjustment;
            if (float.TryParse(s, out v))
                ls.ParamC3Adjustment = v;
            else
                this.Param3Adjust.Text = ls.ParamC3Adjustment.ToString();
        }

        private void Combo_Change_Param3AdjustType(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.ParamC3AdjustmentType = (LSystem_Musical.AdjustmentType)(((ValueDescription)(this.Param3AdjustType.SelectedItem)).Value);
        }

        private void Combo_Change_PitchSource(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.PitchDeterminationType = (LSystem_Musical.IncrementOrAxis)(((ValueDescription)(this.PitchSource.SelectedItem)).Value);
        }

        private void Combo_Change_DurationSource(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.DurationDeterminationType = (LSystem_Musical.IncrementOrAxis)(((ValueDescription)(this.DurationSource.SelectedItem)).Value);
        }

        private void Combo_Change_VolumeSource(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.VolumeDeterminationType = (LSystem_Musical.IncrementOrAxis)(((ValueDescription)(this.VolumeSource.SelectedItem)).Value);
        }

        private void Combo_Change_Param1Source(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.ParamC1DeterminationType = (LSystem_Musical.IncrementOrAxis)(((ValueDescription)(this.Param1Source.SelectedItem)).Value);
        }

        private void Combo_Change_Param2Source(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.ParamC2DeterminationType = (LSystem_Musical.IncrementOrAxis)(((ValueDescription)(this.Param2Source.SelectedItem)).Value);
        }

        private void Combo_Change_Param3Source(object sender, SelectionChangedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.ParamC3DeterminationType = (LSystem_Musical.IncrementOrAxis)(((ValueDescription)(this.Param3Source.SelectedItem)).Value);
        }

        private void Checkbox_Check_AbsAxisVals(object sender, RoutedEventArgs e)
        {
            var ls = (LSystem_Musical)(LSystem.I());
            ls.isAxisValuesAbsolute = useAbsoluteAxisVals.IsChecked.HasValue && useAbsoluteAxisVals.IsChecked.Value;
        }

        
    }
}
