﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SharpGL;
using System.IO;

namespace LindenmayerTool
{
    class ObjExporter
    {
        // Settings

        String Def_Vertex(Vertex v)
        {
            return "v " + v.X + " " + v.Y + " " + v.Z + Environment.NewLine;
        }
        String Def_Face(int start, int count)
        {
            String fd = "f ";
            for (int i = 0; i < count; i++)
                fd += (start + i).ToString() + " ";
            return fd + Environment.NewLine;
        }

        String GenerateObjFile(LSystem lsys)
        {
            StringBuilder sb = new StringBuilder();

            // 1-----0
            //  \     \         OBJ requires
            //   \     \        CCW vertex order
            //    \     \
            //     2-----3

            int v_index = 1;
            foreach (DrawObject.DO_Line l in lsys.Geometry)
            {
                int v_count = 4;
                Vertex[] v = new Vertex[v_count];
                v[0] = new Vertex(l.P2.X + l.Width / 2, l.P2.Y, l.P2.Z);
                v[1] = new Vertex(l.P2.X - l.Width / 2, l.P2.Y, l.P2.Z);
                v[2] = new Vertex(l.P1.X - l.Width / 2, l.P1.Y, l.P1.Z);
                v[3] = new Vertex(l.P1.X + l.Width / 2, l.P1.Y, l.P1.Z);
                for (int i = 0; i < v_count; i++ )
                    sb.Append(Def_Vertex(v[i]));
                sb.Append(Def_Face(v_index, v_count));
                v_index += v_count;
            }

            return sb.ToString();
        }

        public void Export(LSystem lsys, String filename)
        {
            var objString = GenerateObjFile(lsys);
            File.WriteAllText(filename, objString);
        }

    }
}
