﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace LindenmayerTool
{
    public class ProductionRule
    {
        public String Tag;
        // Strings for both sides may represent regex expressions (for parametric systems)
        // RHS is a tuple that includes a weight for probability. (for stochastic systems)
        public String LHS;
        public List<Tuple<float,String>> RHS = new List<Tuple<float,string>>();

        public ProductionRule(string tag)
        {
            Tag = tag;
        }
        public ProductionRule(String lhs, String rhs, float p)
        {
            LHS = lhs;
            RHS.Add(new Tuple<float,string>(p,rhs));
        }

        public Boolean isMatch(String token)
        {
            //Regex matcher = new Regex("$"+LHS+"\\(?\\)?", RegexOptions.None);
            return token.Equals(LHS);
        }

        public String getReplacement(String token)
        {
            if (RHS.Count == 0)
                return LHS;
            else if (RHS.Count == 1)
                return RHS.First().Item2;
            else
            {
                // If multiple replacements are valid, use a random one.
                // ( Find total probability weight, and start going through list of 
                // replacements until the cumulative traversed weight exceeds it).
                float maxProbWeight = 0;
                RHS.ForEach(s => maxProbWeight += s.Item1);
                float randWeightPicked = (float) LSystem.I().RndGen.NextDouble() * maxProbWeight;
                float curProbWeight = 0;
                foreach (var rhs in RHS)
                    if (curProbWeight <= randWeightPicked && randWeightPicked <= curProbWeight + rhs.Item1)
                        return rhs.Item2;
                    else
                        curProbWeight += rhs.Item1;
                return RHS.First().Item2;
            }
        }
    }
}
