﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace LindenmayerTool
{
    /// <summary>
    /// Interaction logic for WrapOptions.xaml
    /// </summary>
    public partial class WrapOptions : Window
    {

        private static WrapOptions _instance = null;
        public static WrapOptions I()
        {
            if (_instance == null) _instance = new WrapOptions();
            if (!_instance.IsLoaded) _instance = new WrapOptions();
            return _instance;
        }


        public enum Axis
        {
            X, Y, Z
        }
        public Axis WrapAxis = Axis.Z;

        public enum OverlapScales
        {
            Units,
            Steps,
            Percent
        }
        public OverlapScales OverlapScale = OverlapScales.Percent;

        public float Overlap = 100;
        public bool IsProjectionScaling = false;
        public bool Is3DRing = false;


        public WrapOptions()
        {
            InitializeComponent();

            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<Axis>())
            {
                this.ComboWrapAxis.Items.Add(i);
                if (WrapAxis == (Axis)i.Value)
                    ComboWrapAxis.SelectedItem = i;
            }
            foreach (var i in EnumHelper.GetAllValuesAndDescriptions<OverlapScales>())
            {
                this.ComboOverlapScale.Items.Add(i);
                if (OverlapScale == (OverlapScales)i.Value)
                    ComboOverlapScale.SelectedItem = i;
            }
            this.TextOverlapAmount.Text = Overlap.ToString();
            this.ChkboxProjectionScaling.IsChecked = IsProjectionScaling;
            this.Chkbox3DRing.IsChecked = Is3DRing;
        }

        private void Combo_Change_WrapAxis(object sender, SelectionChangedEventArgs e)
        {
            this.WrapAxis = (Axis)(((ValueDescription)(this.ComboWrapAxis.SelectedItem)).Value);
        }
        private void Combo_Change_OverlapScale(object sender, SelectionChangedEventArgs e)
        {
            this.OverlapScale = (OverlapScales)(((ValueDescription)(this.ComboOverlapScale.SelectedItem)).Value);
        }
        private void Text_Change_OverlapAmount(object sender, TextChangedEventArgs e)
        {
            String sVal = ((TextBox)sender).Text;
            Double dVal;
            if (Double.TryParse(sVal, out dVal))
                Overlap = (float)dVal;
            else
                ((TextBox)sender).Text = Overlap.ToString();
        }
        private void Chkbox_Check_ProjectionScaling(object sender, RoutedEventArgs e)
        {
            IsProjectionScaling = ((CheckBox)sender).IsChecked ?? false;
        }
        private void Chkbox_Check_3DRing(object sender, RoutedEventArgs e)
        {
            Is3DRing = ((CheckBox)sender).IsChecked ?? false;
        }
        private void Button_Click_Cancel(object sender, RoutedEventArgs e)
        {
            var mb = MessageBox.Show("Keep wrap settings?","Close", MessageBoxButton.YesNo);
            if (mb != MessageBoxResult.Yes)
                LSystem.I().UnApplyWrap();

            this.Hide();
        }
        private void Button_Click_Apply(object sender, RoutedEventArgs e)
        {
            LSystem.I().ApplyWrap(WrapAxis, OverlapScale, Overlap, IsProjectionScaling);
        }
        


    }
}
